﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

namespace GGJ2015LevelCreator
{
    class Program
    {
        static Troschuetz.Random.IGenerator Randomiser = new Troschuetz.Random.Generators.MT19937Generator();
        
        static void Main(string[] args)
        {
            int maxWidth = 32;
            int maxHeight = 12;

            var runDateTime = DateTime.Now;

            var rooms = new List<string[]>();
            for (int i = 0; i < 3; i++)
            {
                var iterationNumber = 1;
                var room = GenerateLevel(maxWidth, maxHeight, iterationNumber);

                rooms.Add(room);
            }

            var roomNames = new List<string>();
            foreach (var room in rooms) 
            {
                var name = string.Empty;

                foreach (var row in room) { 

                    if (row.Contains(Tiles.Water) && !name.Contains(Tiles.Water))
                    {
                        name += Tiles.Water;
                    }

                    if (row.Contains(Tiles.Damageable) && !name.Contains(Tiles.Damageable))
                    {
                        name += Tiles.Damageable;
                    }

                    if (row.Contains(Tiles.Start) && !name.Contains(Tiles.Start))
                    {
                        name = Tiles.Start + name;
                    }
                    else if (row.Contains(Tiles.Finish) && !name.Contains(Tiles.Finish))
                    {
                        name = Tiles.Finish + name;
                    }

                    Console.WriteLine(row);

                }
                if (!name.Contains(Tiles.Start) && !name.Contains(Tiles.Finish))
                {
                    name = "I" + name;
                }

                name += "-";              
                var roomNumber = roomNames.Count(rn => rn.StartsWith(name)) + 1;

                name += roomNumber;
                roomNames.Add(name);

                Console.WriteLine(name);
                Console.WriteLine();

                var directoryName = string.Format("C:/GGJ2015Levels/Levels-{0}", runDateTime.ToString("ddMMyyyyhhmmss"));

                if (!Directory.Exists(directoryName))
                    Directory.CreateDirectory(directoryName);

                File.WriteAllLines(string.Format("{0}/{1}.txt", directoryName, name), room);
            }

            Console.ReadLine();
        }

        private static string[] GenerateLevel(int levelWidth, int levelHeight, int iterationNumber)
        {
            var usedLevelHeight = Randomiser.Next(4, levelHeight);
            var usedLevelWidth = Randomiser.Next(8, levelWidth);

            var blockLimit = Randomiser.Next(usedLevelHeight * usedLevelWidth / 4);

            var usingWater = Randomiser.NextBoolean();
            var waterLimit = usingWater ? Randomiser.Next(usedLevelHeight * usedLevelWidth / 4) : 0;

            var usingDirt = Randomiser.NextBoolean() || !usingWater; 
            var dirtLimit = usingDirt ? Randomiser.Next(usedLevelHeight * usedLevelWidth / 4) : 0;

            var createStartPoint = false;
            var createEntrance = false;

            var createExit = false;
            var createFinishPoint = false;

            var roomType = Randomiser.Next(1, 4);

            switch (roomType) 
            { 
                case 1:
                    createStartPoint = true;
                    createExit = true;
                    break;
                case 2:
                    createEntrance = true;
                    createExit = true;
                    break;
                case 3:
                    createEntrance = true;
                    createFinishPoint = true;
                    break;   
            }

            var addedStartPoint = false;
            var addedEntrance = false;
            var addedFinishPoint = false;
            var addedExitPoint = false;

            int startPositionX = -1;
            int startPositionY = -1;
            int entryPositionX = -1;
            int entryPositionY = -1;
            int finishPositionX = -1;
            int finishPositionY = -1;
            int exitPositionX = -1;
            int exitPositionY = -1;

            char[,] levelTiles = new char[usedLevelHeight, usedLevelWidth];
            var airCount = 0;

            // Air pass!
            for (int i = 0; i < usedLevelHeight; i++)
            {
                for (int j = 0; j < usedLevelWidth; j++)
                {
                    if (i == 0 || j == 0 || i == usedLevelHeight - 1 || j == usedLevelWidth - 1)
                    {
                        levelTiles[i, j] = Tiles.Boundary;
                        continue;
                    }
                    
                    levelTiles[i, j] = Tiles.Air;
                    airCount++;
                }

            }

            ////Terrain pass
            var dirtCount = 0;
            while (dirtCount < dirtLimit)
            {
                var randomXPosition = Randomiser.Next(1, usedLevelWidth - 1);
                var randomYPosition = Randomiser.Next(1, usedLevelHeight - 1);
                if (levelTiles[randomYPosition, randomXPosition] == Tiles.Air)
                {
                    levelTiles[randomYPosition, randomXPosition] = Tiles.Damageable;
                    airCount--;
                    dirtCount++;
                }

                if (airCount == 0)
                    break;
            }

            var blockCount = 0;
            while (blockCount < blockLimit)
            {
                var randomXPosition = Randomiser.Next(1, usedLevelWidth - 1);
                var randomYPosition = Randomiser.Next(1, usedLevelHeight - 1);
                if (levelTiles[randomYPosition, randomXPosition] == Tiles.Air)
                {
                    levelTiles[randomYPosition, randomXPosition] = Tiles.Block;
                    airCount--;
                    blockCount++;
                }

                if (airCount == 0)
                    break;
            }

            var waterCount = 0;
            while (waterCount < waterLimit)
            {
                var randomXPosition = Randomiser.Next(1, usedLevelWidth - 1);
                var randomYPosition = Randomiser.Next(1, usedLevelHeight);
                if ((randomYPosition == usedLevelHeight - 1 && randomXPosition != 0 && randomXPosition != usedLevelWidth - 1 ) || (levelTiles[randomYPosition + 1, randomXPosition].IsInvinskible() && levelTiles[randomYPosition, randomXPosition - 1].IsInvinskible() && levelTiles[randomYPosition, randomXPosition + 1].IsInvinskible()))
                {
                    levelTiles[randomYPosition, randomXPosition] = Tiles.Water;
                    waterCount++;
                }
            }

            if (createStartPoint)
            {
                while (!addedStartPoint)
                {
                    var randomXPosition = Randomiser.Next(1, usedLevelWidth - 1);
                    var randomYPosition = Randomiser.Next(1, usedLevelHeight - 1);

                    if (levelTiles[randomYPosition, randomXPosition] == Tiles.Air)
                    {
                        levelTiles[randomYPosition, randomXPosition] = Tiles.Start;
                        addedStartPoint = true;
                        startPositionX = randomXPosition;
                        startPositionY = randomYPosition;
                    }
                }
            }
            

            if (createEntrance)
            {
                while (!addedEntrance)
                {
                    var randomXPosition = 0;
                    var randomYPosition = Randomiser.Next(0, usedLevelHeight - 1);

                    if (levelTiles[randomYPosition, randomXPosition] == Tiles.Boundary && levelTiles[randomYPosition + 1, randomXPosition + 1].IsSolid() && levelTiles[randomYPosition, randomXPosition + 1].IsPassable())
                    {
                        levelTiles[randomYPosition, randomXPosition] = Tiles.Entrance;
                        addedEntrance = true;
                        entryPositionX = randomXPosition;
                        entryPositionY = randomYPosition;
                    }
                }
            }

           if (createExit) {
               while (!addedExitPoint)
               {
                   var randomXPosition = usedLevelWidth - 1;
                   var randomYPosition = Randomiser.Next(0, usedLevelHeight - 1);

                   if (levelTiles[randomYPosition, randomXPosition] == Tiles.Boundary 
                       && levelTiles[randomYPosition + 1, randomXPosition - 1].IsSolid() 
                       && levelTiles[randomYPosition, randomXPosition - 1].IsPassable())
                   {
                       levelTiles[randomYPosition, randomXPosition] = Tiles.Exit;
                       addedExitPoint = true;
                       exitPositionX = randomXPosition;
                       exitPositionY = randomYPosition;
                   }
               }
           }

           if (createFinishPoint) {
               while (!addedFinishPoint)
               {
                   var randomXPosition = Randomiser.Next(0, usedLevelWidth - 1);
                   var randomYPosition = Randomiser.Next(0, usedLevelHeight - 1);

                   if (levelTiles[randomYPosition, randomXPosition] == Tiles.Air)
                   {
                       levelTiles[randomYPosition, randomXPosition] = Tiles.Finish;
                       addedFinishPoint = true;
                       finishPositionX = randomXPosition;
                       finishPositionY = randomYPosition;
                       break;
                   }
               } 
           }

            var levelRows =  new List<string>();

            for (int i = 0; i < usedLevelHeight; i++)
            {
                var levelRow = string.Empty;
                for (int j = 0; j < usedLevelWidth; j++)
                {
                    levelRow += levelTiles[i, j];
                }

                levelRows.Add(levelRow);
            }

            if ((createEntrance && levelRows.All(r => !r.Contains(Tiles.Entrance))
                || createExit && levelRows.All(r => !r.Contains(Tiles.Exit))
                || createFinishPoint && levelRows.All(r => !r.Contains(Tiles.Finish))
                || createStartPoint && levelRows.All(r => !r.Contains(Tiles.Start))))
                return GenerateLevel(levelWidth, levelHeight, ++iterationNumber);

            Console.WriteLine("Generated a usable room after {0} iterations", iterationNumber);
            return levelRows.ToArray();
        }
    }
}
