#include <cstdlib>

#include "Game.hpp"
#include "Constants.hpp"
#include "Window.hpp"
#include "Input.hpp"
#include "Player.hpp"
#include "Sound.hpp"
#include "world.hpp"

void Game::init() {
	WINDOW.open("Potentia", 640, 448);//, SDL_WINDOW_FULLSCREEN_DESKTOP);

	you_.init();
	World::instance().init(10);
	won_ = false;
	World::instance().new_room();
}

void Game::run() {
	if(!intro() || you_.pos().pos().x < 0.5f)
		return;

	bool repeat(false);

	do {
		World::instance().setRoom(World::instance().getRoom(0));
		you_.update();
		WINDOW.flush();

		while(!INPUT.quit() && !(you_.dead() && WINDOW.faded()) && !Game::instance().have_won()) {
			INPUT.scan();

			World::instance().update();
			you_.update();
			#ifndef __APPLE__
		    		AUDIO.update();
			#endif

			World::instance().draw();
			you_.draw();

			WINDOW.flush();
		}

		repeat = outro();
		if(repeat) {
			World::instance().init(10);
			World::instance().new_room();
		}
	} while(!INPUT.quit() && repeat);
}

bool Game::intro()
{
	you_.resurrect();
	Room openingMenu("levels/Menu.txt");
	World::instance().setRoom(&openingMenu);

	GLuint quitTex(GFX_POOL.loadImage("gfx/Quit_Sign.png"));
	GLuint contTex(GFX_POOL.loadImage("gfx/Start_Sign.png"));

	GLuint bgTex(GFX_POOL.loadImage("gfx/Title_Tile.png"));
	WINDOW.draw(bgTex, Vector(), Vector(1.0f, 1.0f));

	#ifndef __APPLE__
        	AUDIO.open("audio/musicfinal.wav", true);
	#endif

	while(!INPUT.quit()) {
		INPUT.scan();

		if(WINDOW.faded())
			return true;

		World::instance().update();
		you_.update();
		#ifndef __APPLE__
            		AUDIO.update();
        	#endif

		World::instance().draw();
		you_.draw();

		WINDOW.draw(quitTex, Vector(0.0f, 0.5f), V64x64);
		WINDOW.draw(contTex, Vector(0.9f, 0.5f), V64x64);

		WINDOW.flush();
		WINDOW.draw(bgTex, Vector(), Vector(1.0f, 1.0f));
	}

	return false;
}

bool Game::outro()
{
	you_.resurrect();
	Room outRoom("levels/Retry.txt");
	World::instance().setRoom(&outRoom);
	you_.pos().setPos(Vector(0.5f, 0.5f));
	WINDOW.flush();

	if(you_.dead())
		printf("Game over...\n");
  	else if(Game::instance().have_won())
    		std::cout << "You won!" << std::endl;

	while(!INPUT.quit() && !(CURRENT_ROOM.getProps()->size() < 2 && WINDOW.faded())) {
		INPUT.scan();

		World::instance().update();
		you_.update();

		if(CURRENT_ROOM.getProps()->size() < 2)
			WINDOW.fadeOut();

		World::instance().draw();
		you_.draw();

		WINDOW.print("Quit", Vector(0.2f, 0.6f), WHITE);
		WINDOW.print("Retry", Vector(0.7f, 0.6f), WHITE);

		WINDOW.flush();
	}

	return CURRENT_ROOM.getProps()->at(0).code() == 'y';
}
