#include "Position.hpp"
#include "world.hpp"

Vector Vector::operator+(const Vector& add) const
{
  return Vector(x + add.x, y + add.y);
}


Vector Vector::operator-(const Vector& sub) const
{
  return Vector(x - sub.x, y - sub.y);
}


Vector& Vector::operator+=(const Vector& add)
{
  this->x += add.x;
  this->y += add.y;
  return *this;
}


Vector& Vector::operator-=(const Vector& sub)
{
  this->x -= sub.x;
  this->y -= sub.y;
  return *this;
}


Vector Vector::operator*(const float fct) const
{
  return Vector(fct * this->x, fct * this->y);
}


Vector Vector::operator/(const float fct) const
{
  return Vector(this->x / fct, this->y / fct);
}


Vector& Vector::operator*=(const float fct)
{
  this->x *= fct;
  this->y *= fct;
  return *this;
}


Vector& Vector::operator/=(const float fct)
{
  this->x /= fct;
  this->y /= fct;
  return *this;
}


std::ostream& operator<<(std::ostream& out, Vector& vec)
{
  out << "(" << vec.x << ", " << vec.y << ")";
  return out;
}


void Position::init(Vector pos, Vector vel, Vector acc) {
	pos_ = pos;

	set(vel, acc);
}

//Quick little function for reducing code bulk.
bool canMoveHere(const Vector& base, const Vector& off,
  const Vector& ins)
{
  return (base.x + off.x >= 0.0f && base.x + ins.x < 1.0f
    and CURRENT_ROOM.can_move_here(base + off)
    and CURRENT_ROOM.can_move_here(base + Vector(off.x, ins.y))
    and CURRENT_ROOM.can_move_here(base + ins)
    and CURRENT_ROOM.can_move_here(base + Vector(ins.x, off.y)));
}

void Position::update(float vDecay, Vector offset, Vector inset)
{
	Vector oldPos(pos_);

	if(canMoveHere(pos_ + Vector(vel_.x, 0.0f), offset, inset)) {
		pos_.x += vel_.x / CURRENT_ROOM.get_block(pos_).get_viscosity();
		vel_.x += acc_.x;
	}
	if(canMoveHere(oldPos + Vector(0.0f, vel_.y), offset, inset)) {
		pos_.y += vel_.y / CURRENT_ROOM.get_block(oldPos).get_viscosity();
		vel_.y += acc_.y;
	}

	//Decay the velocity (friction).
	vel_ *= vDecay;

	if(fabs(vel_.x) <= SMALL_FLOAT)
		vel_.x = 0.0f;
	if(fabs(vel_.y) <= SMALL_FLOAT)
		vel_.y = 0.0f;
}

void Position::set(Vector vel, Vector acc)
{
  vel_ = vel;
  acc_ = acc;
}
