#ifndef POSITION_HPP_INCLUDED
#define POSITION_HPP_INCLUDED

#include <math.h>
#include "Constants.hpp"

class World;

//A generic Vector class.
struct Vector {
	float x;
	float y;

	Vector (float xIn = 0.0f, float yIn = 0.0f) : x(xIn), y(yIn) {}

	Vector operator+(const Vector& add) const;
	Vector& operator+=(const Vector& add);
	Vector operator-(const Vector& sub) const;
	Vector& operator-=(const Vector& sub);

	Vector operator*(float fct) const;
	Vector& operator*=(float fct);
	Vector operator/(float fct) const;
	Vector& operator/=(float fct);

  float mag() const { return float(sqrt(x * x + y * y)); }
  Vector norm() const { return Vector(*this) / this->mag(); }

  friend std::ostream& operator<<(std::ostream& out, Vector& vec);
};

#define V12x12 Vector(WINDOW.floatifyX(12),WINDOW.floatifyY(12))
#define V64x64 Vector(WINDOW.floatifyX(64),WINDOW.floatifyY(64))

//An equivalent to a vector class, but with added capabilities.
class Position
{
	public:
	//Constructors.
		Position () { init(Vector(), Vector(), Vector()); }
		Position (Vector pos, Vector vel = Vector(), Vector acc = Vector()) { init(pos, vel, acc); }
	//Destructor
		~Position() {}

	//The initialising function.
		void init(Vector pos, Vector vel = Vector(), Vector acc = Vector());

	//The function to update the position.
		void update(float vDecay = VELOCITY_DECAY, Vector offset = Vector(), Vector inset = Vector());

	//Functions for getting the position and velocity.
		Vector& pos() { return pos_; }
    		const Vector& pos() const { return pos_; }
		Vector& vel() { return vel_; }
		Vector& acc() { return acc_; }

	//Sets all of the data.
		void set(Vector vel = Vector(), Vector acc = Vector());

	//Lets you set data by aiming it at a point. Doesn't set acceleration.
		void aim(Vector pos, int t) { set(t, (pos - pos_) / float(t)); }

	//Forces the position somewhere. It doesn't affect any of the other values though.
		void setPos(Vector pos) { pos_ = pos; }

	//A function to add a velocity to the current one.
		void addVel(Vector vel) { vel_ += vel; }
		void setVel(Vector vel) { vel_ = vel; }

	//For setting the acceleration.
		void setAcc(Vector set) { acc_ = set; }

	//For casting to a vector. It only considers the position part.
		operator Vector() { return pos_; }

	protected:
		Vector pos_;

		Vector vel_;
		Vector acc_;

	private:
};

#endif // POSITION_HPP_INCLUDED
