#ifndef PROP_HPP_INCLUDED
#define PROP_HPP_INCLUDED

#include "PropList.hpp"
#include "Position.hpp"
#include "Sprite.hpp"
#include "Game.hpp"

class Prop
{
	public:
		Prop () {}
		Prop (PropData data, Vector pos) { init(data, pos); }
		~Prop() {}

		void init(PropData data, Vector pos) {
			name_ = data.name;
			code_ = data.code;
			pos_ = Position(pos, Vector(), Vector(0.0f, data.obeyGravity ? GRAVITY : 0.0f));

			sprite_.init("gfx/Placeholder_Props.png", V64x64, Vector(0.0f, data.yIndex / 16.0f), Vector(0.25f, 1.0f / 16.0f));

			lethal_ = data.lethal;
			solid_ = data.solid;
			wantedEffects_ = data.wantedEffects;

			result_ = RES_NONE;
		}

		void update() {
			pos_.update(VELOCITY_DECAY, Vector(SMALL_FLOAT, SMALL_FLOAT), sprite_.dim() - Vector(SMALL_FLOAT, SMALL_FLOAT));
			sprite_.update();

			if(lethal_ && inside(Game::instance().getGuy()->pos().pos()))
				Game::instance().getGuy()->kill();

			if(changed()) {
				sprite_.setFrames(1);
				sprite_.texPos().x = 2.0f * sprite_.texDim().x;

				if(result_ == RES_ADD_GRAV) {
					if(code_ == 'l')
						pos_.set(Vector(), Vector(0.0f, GRAVITY));
					else if(code_ == 'd')
						pos_.set(Vector(), Vector(0.0f, -1.0f * GRAVITY));

					result_ = RES_NONE;
				}
				else if(result_ == RES_BORING)
					result_ = RES_NONE;
			}
		}

		std::string name() { return name_; }
		char code() { return code_; }

		void draw() { sprite_.draw(pos_); }

		Sprite& sprite() { return sprite_; }

		Position& pos() { return pos_; }
		bool inside(Vector pos) { return pos.x >= pos_.pos().x && pos.x < pos_.pos().x + sprite_.dim().x && pos.y >= pos_.pos().y && pos.y < pos_.pos().y + sprite_.dim().y; }

		bool solid() { return solid_; }
		void solidify() { solid_ = true; }

		bool wantsEffect(int effect) { return effect & wantedEffects_; }
		int wantedEffects() { return wantedEffects_; }

		void change(int result) { sprite_.setFrames(3); result_ = result; }
		bool changed() { return result_ != RES_NONE && sprite_.currentFrame() == sprite_.frames() - 1; }

		int result() { return result_; }

	protected:
		std::string name_;
		char code_;
		Position pos_;

		Sprite sprite_;

		bool lethal_;
		bool solid_;
		int wantedEffects_;

		int result_;

	private:
};

#endif
