#ifndef SOUND_HPP_INCLUDE
#define SOUND_HPP_INCLUDE

#include <alut.h>

class Sound
{
	public:
		~Sound() {
			//Delete the sources and buffers.
			for(int index(0); index < int(data_.size()); ++index)
				remove(index);

			//Then turn the audio off.
			alutExit();
		}

		static Sound& instance() {
			static Sound pass;
			return pass;
		}

		struct AudioData {
			ALuint buffer;
			ALuint source;

			bool loop;
		};

		int open(std::string fileName, bool loop = false) {
			if(fileName!= "") {
				//Creating the buffer.
				AudioData data;

				alGenBuffers(1, &data.buffer);

				//Loading an audio file into a buffer.
				data.buffer = alutCreateBufferFromFile(fileName.c_str());

				//Generate the sources
				alGenSources(1, &data.source);

				//Attaching a buffer to a source.
				alSourcei(data.source, AL_BUFFER, data.buffer);

				//Set the volume.
				alSourcef(data.source, AL_GAIN, 1.0f);

				data.loop = loop;
				data_.push_back(data);

				play(data_.size() - 1);
				return data_.size() - 1;
			}
			else
				return -1;
		}

		void play(int index) { alSourcePlay(data_[index].source); }
		void pause(int index) { alSourcePause(data_[index].source); }
		void stop(int index) { alSourceStop(data_[index].source); }

		void remove(int index) {
			alDeleteSources(1, &data_[index].source);
			alDeleteBuffers(1, &data_[index].buffer);
		}

		void setVolume(int index, float set) { alSourcef(data_[index].source, AL_GAIN, set); }

		void update() {
			//Check to see if it's stopped.
			int state(0);
			for(std::vector<AudioData>::iterator datum(data_.begin()); datum < data_.end();) {
				alGetSourcei(datum->source, AL_SOURCE_STATE, &state);

				//If it has stopped, start it up again.
				if(state == AL_STOPPED) {
					if(datum->loop) {
						play(datum - data_.begin());
						++datum;
					}
					else {
						remove(datum - data_.begin());
						datum = data_.erase(datum);
					}
				}
				else
					++datum;
			}
		}

	protected:
		Sound () {
			//Init openAL
			alutInit(0, NULL);

			//Clear Error Code (so we can catch any new errors).
			alGetError();
		}

		std::vector<AudioData> data_;

	private:
};

#define AUDIO Sound::instance()

#endif
