#ifndef SPRITE_HPP_INCLUDE
#define SPRITE_HPP_INCLUDE

#include <string>
#include "GraphicPool.hpp"
#include "Window.hpp"
#include "Position.hpp"

//A general sprite class. Animates the sprite on each update.

class Sprite
{
	public:
	//Constructors and destructor.
		Sprite () {}
		Sprite (std::string fileName, Vector dim, Vector texPos, Vector texDim, int frames = 1)
			{ init(fileName, dim, texPos, texDim, frames); }
		~Sprite() {}

	//The function to initialise everything. Should be called before anything else is done.
		void init(std::string fileName, Vector dim, Vector texPos, Vector texDim, int frames = 1) {
			tex_ = GFX_POOL.loadImage(fileName);

			dim_ = dim;

			texPos_ = texPos;
			texDim_ = texDim;

			frames_ = frames;
			current_ = 0;
			delay_ = FRAME_COOL_DOWN;

			step_ = 1;
		}

	//Functions to access the position-esque parts.
		Vector& dim() { return dim_; }

		Vector& texPos() { return texPos_; }
		Vector& texDim() { return texDim_; }

	//Function for setting the number of frames.
		void setFrames(int set) {
			frames_ = set;
			current_ = current_ % frames_;
		}

		int currentFrame() { return current_; }
		int frames() { return frames_; }

	//Function for forcing the sprite across on it's sheet by a set amount.
		void shunt(int amount) { texPos_.x += texDim_.x * amount; }

	//The function that updates the sprite's position on the sprite sheet.
		void update() {
			if(delay_-- <= 0) {
				current_ = (current_ + frames_ + step_) % frames_;
				delay_ = FRAME_COOL_DOWN;
			}
		}

	//Reverses the direction the sprite animates in.
		void reverse() { step_ = -1 * step_; }
		int step() { return step_; }

	//The function that draws the sprite onto the screen between the given set of corners.
		void draw(Vector p1, Vector p2, Vector p3, Vector p4) { WINDOW.draw(tex_, p1, p2, p3, p4, texPos_ + Vector(texDim_.x * current_, 0.0f), texDim_); }

	//The function that draws the sprite onto the screen at the given position.
		void draw(Vector pos) { WINDOW.draw(tex_, pos, dim_, texPos_ + Vector(texDim_.x * current_, 0.0f), texDim_); }

	protected:
		GLuint tex_;

		Vector dim_;

		Vector texPos_;
		Vector texDim_;

		int frames_;
		int current_;
		int delay_;

		int step_;

	private:
};

#endif
