#ifndef BLOCK_HPP
#define BLOCK_HPP

#include "Position.hpp"
#include "PropList.hpp"
#include "Sprite.hpp"

class Block
{
public:
  Block(const Position& position)
    : position_(position), viscosity_(1.0f), is_traversable_(true), is_exit_(false), is_entrance_(false),
      wantedEffects_(EFF_NONE), index_(-1)
  {  }
  virtual ~Block() { }

  Vector& pos() { return position_.pos(); }
  const Vector& pos() const { return position_.pos(); }
  const float get_viscosity() { return viscosity_; }
  bool is_traversable() const { return is_traversable_; }

  bool wantsEffect(int effect) { return effect & wantedEffects_; }
	int wantedEffects() { return wantedEffects_; }

  void make_exit() { is_exit_ = true; }
	bool is_exit() { return is_exit_; }

  int sprite_index() { return index_; }

	Sprite& sprite() { return sprite_; }
	void setSprite(int index) { sprite_.init("gfx/Placeholder_Tileset.png", V64x64, Vector(0, index / 16.0), Vector(1.0 / 16, 1.0 / 16)); }

protected:
  Position position_;
	Sprite sprite_;

  float viscosity_;
  bool is_traversable_;
  bool is_exit_;
  bool is_entrance_;

  int wantedEffects_;
  int index_;
};


class AirBlock : public Block
{
public:
  AirBlock(const Position& position) : Block(position)
  { viscosity_ = 1.0f; setSprite(index_); }
};


class DirtBlock : public Block
{
public:
  DirtBlock(const Position& position) : Block(position)
  { viscosity_ = 1.0; is_traversable_ = false; wantedEffects_ = EFF_DIG; index_ = 1; setSprite(index_); }
};


class BoundaryBlock : public Block
{
public:
  BoundaryBlock(const Position& position) : Block(position)
  { viscosity_ = 1.0f; is_traversable_ = false; index_ = 0; setSprite(index_); }
};


class WaterBlock : public Block
{
public:
  WaterBlock(const Position& position) : Block(position)
    { viscosity_ = 10.0f; wantedEffects_ = EFF_FREEZE; index_ = 2; setSprite(index_); }
};

class IceBlock : public Block
{
public:
    IceBlock(const Position& position) : Block(position)
    { viscosity_ = 1.0; is_traversable_ = false; wantedEffects_ = EFF_DIG | EFF_BREAK; index_ = 3; setSprite(index_); }
};


class EntranceBlock : public Block
{
public:
  EntranceBlock(const Position& position) : Block(position)
  { viscosity_ = 1.0f; is_entrance_ = true; setSprite(index_); }
};


class ExitBlock : public Block
{
public:
  ExitBlock(const Position& position) : Block(position)
  { viscosity_ = 1.0f; is_exit_ = true; setSprite(index_); }
};

#endif
