#ifndef ROOM_HPP
#define ROOM_HPP

#include <algorithm>
#include <iostream>
#include <memory>
#include <vector>

#include "Prop.hpp"
#include "block.hpp"

class Room
{
public:
  typedef std::vector<std::unique_ptr<Block> > BlockArray;

  Room() : empty_(Vector()) { };
  Room(const std::string& path) : empty_(Vector()) {
    make_from_file(path);
  };
  Room(Room&& room);
  ~Room() { };

  Room& operator=(Room&& room);

  void make_from_file(const std::string& path);

  template <typename T>
  void add_block(const T& block_ptr);
  template <typename T>
  void replace_block(const int index);

	void applyToBlock(int index, int effect, const Vector& vec = Vector());
	void applyToProp(int index, int effect, const Vector& vec = Vector());

  const Block& get_block(const Position& coordinates) const;
  Block& get_block(const Position& coordinates);
	Prop* getProp(Position coords);
  int get_block_index(const Position& coordinates);
  int get_prop_index(Position coords);
  const BlockArray& get_blocks() const { return blocks_; }
	std::vector<Prop>* getProps() { return &props_; }

  bool can_move_here(const Position& coordinates);

  float get_block_width() const { return block_width_; }
  float get_block_height() const { return block_height_; }
    
  int tiling_index(int i);

	void update();
	void draw();

	Block* get_exit_block() { return exit_block_; }
	Block* get_entry_block() { return entry_block_; }

private:

  typedef std::unique_ptr<Block> BlockPtr;

  BlockArray blocks_;
  float block_width_;
  float block_height_;
  Block* exit_block_;
  Block* entry_block_;
  std::vector<std::vector<int> > random_blocks_;

	Prop gate_;
	bool gateInRoom_;

  Block empty_;

	std::vector<Prop> props_;
	std::vector<Prop>::iterator propIter_;
  
  bool check_adjacent(int checkID, int x, int y);
    
};

#endif
